/*

     d888   8888888888     d88P 8888888888     d88P  .d8888b.  888888888
    d8888         d88P    d88P        d88P    d88P  d88P  Y88b 888
      888        d88P    d88P        d88P    d88P   888    888 888
      888       d88P    d88P        d88P    d88P    Y88b. d888 8888888b.
      888    88888888  d88P      88888888  d88P      "Y888P888      "Y88b
      888     d88P    d88P        d88P    d88P             888        888
      888    d88P    d88P        d88P    d88P       Y88b  d88P Y88b  d88P
    8888888 d88P    d88P        d88P    d88P         "Y8888P"   "Y8888P"

*/

/*** BEZIER.C ***/

typedef struct{ float x, y, z; } t_ponto;

#define MAX_PONTOS 10
t_ponto p1[MAX_PONTOS][MAX_PONTOS], p2[MAX_PONTOS][MAX_PONTOS];
int m, n;

#define X_ELEM(M, i, j) (M[i][j]).x
#define Y_ELEM(M, i, j) (M[i][j]).y
#define Z_ELEM(M, i, j) (M[i][j]).z

#define DEF_PONTO(P, M, N, X, Y, Z)   P[M][N].x=X; P[M][N].y=Y; P[M][N].z=Z

/* ------------------------------------------------------------------------ */

float Bin(int m, int n) {

/* Calcula m!/(n!(m - n)!) = (m(m - 1)...(m - n + 1))/(n(n - 1)...1) */

  float i, prod = 1.0;

  if (m < 0 || n < 0 || m < n) return 0.0;
  if (n > m - n) n = m - n;

  for (i = 1.0; i <= n; i++) prod *= (m - n + i)/i;

  return prod;
}

/* ------------------------------------------------------------------------ */

float pot(float a, float b) {

  if (fabs(a) < 0.001 && fabs(b) < 0.001) return 1.0;
  else return pow(a, b);
}

/* ------------------------------------------------------------------------ */

float Bernstein(int a, int b, float x) {

/* Calcula o valor do polinomio de Bernstein

				  a        b - a
	     B   (x) = Bin(b, a) x  (1 - x)
	      a,b

   x deve estar no intervalo [0, 1]. */

  if (x < 0.0 || x > 1.0) return 0.0;

  return (Bin(b, a)*pot(x, a)*pot(1.0 - x, b - a));
}

/* ------------------------------------------------------------------------ */

float X_Bezier(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n, float u, float v) {

/* Calcula a parametrizacao da coordenada X da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++)
      soma += X_ELEM(P, i, j)*Bernstein(i, m, u)*Bernstein(j, n, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

float Y_Bezier(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n, float u, float v) {

/* Calcula a parametrizacao da coordenada Y da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++)
      soma += Y_ELEM(P, i, j)*Bernstein(i, m, u)*Bernstein(j, n, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

float Z_Bezier(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n, float u, float v) {

/* Calcula a parametrizacao da coordenada Z da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++)
      soma += Z_ELEM(P, i, j)*Bernstein(i, m, u)*Bernstein(j, n, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

/*** FIM DE BEZIER.C ***/

