/*

     d888   8888888888     d88P 8888888888     d88P  .d8888b.  888888888
    d8888         d88P    d88P        d88P    d88P  d88P  Y88b 888
      888        d88P    d88P        d88P    d88P   888    888 888
      888       d88P    d88P        d88P    d88P    Y88b. d888 8888888b.
      888    88888888  d88P      88888888  d88P      "Y888P888      "Y88b
      888     d88P    d88P        d88P    d88P             888        888
      888    d88P    d88P        d88P    d88P       Y88b  d88P Y88b  d88P
    8888888 d88P    d88P        d88P    d88P         "Y8888P"   "Y8888P"

*/

/*** BEZ_DEMO.C - Desenha uma superficie de Bezier, permitindo movimentos
		  para cima ou para baixo dos pontos de controle ***/

#include <math.h>
#include <graphics.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>

#define float long double
#define CAMINHO_BGI   ""
int cor1 = BLUE, cor2 = LIGHTRED;

#include "bezier.c"
#include "graficos.c"

float f1(float u, float v) { return X_Bezier(p1, m, n, u, v); }
float f2(float u, float v) { return Y_Bezier(p1, m, n, u, v); }
float f3(float u, float v) { return Z_Bezier(p1, m, n, u, v); }

/* ------------------------------------------------------------------------ */

void MarcaPontoControle(float x, float y, float z, int cor) {

  float X, Y;

  ProjetaPonto(x, y, z, &X, &Y);
  putpixel((int) (A + X*TAM), (int) (B - Y*TAM), cor);
  putpixel((int) (A + X*TAM + 1), (int) (B - Y*TAM + 1), cor);
  putpixel((int) (A + X*TAM + 1), (int) (B - Y*TAM - 1), cor);
  putpixel((int) (A + X*TAM - 1), (int) (B - Y*TAM + 1), cor);
  putpixel((int) (A + X*TAM - 1), (int) (B - Y*TAM - 1), cor);
}

/* ------------------------------------------------------------------------ */

void MarcaPontosDeControle(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j;

  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++)
      MarcaPontoControle(X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j),
		  cor2);

}

/* ------------------------------------------------------------------------ */

void ImprimePontosCtrl(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j;

  printf("\n\nOs ultimos pontos de controle mostrados sao:\n");
  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++)
      printf("\n    P[%d][%d] = (%5.1Lf, %5.1Lf, %5.1Lf)",
		   i, j,  X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j));
  printf("\n\n");
}

/* ------------------------------------------------------------------------ */

int  MostraNumeros(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j, cont = 0;
  float X, Y;
  char mens[10];

  for (i = 0; i <= m; i++)
    for (j = 0; j <= n; j++) {
      cont++;

      ProjetaPonto(X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j), &X, &Y);
      sprintf(mens, "%d", cont);
      outtextxy((int) (A + X*TAM), (int) (B - Y*TAM), mens);
  }
  outtextxy(1, getmaxy() - 3*textheight("["),
			 " \x10 Pressione Ctrl-Fn (\x19) ou Alt-Fn (\x18) "
			 "para mover o ponto de controle n");
  return 0;
}

/* ------------------------------------------------------------------------ */

void MensagemInicial(void) {

  printf("\n\n*** ESBOO DE UMA SUPERFICIE DE BEZIER COM 9 PONTOS DE CONTROLE ***");
  printf("\n\n  ESC: encerra o programa");
  printf("\n  PgUp: aproxima o grafico         PgDn: afasta o grafico");
  printf("\n  Teclas de setas: movimentam a origem (0, 0, 0) na tela");
  printf("\n  F1: mostra eixos                 F3: muda cor do grafico");
  printf("\n  F2: mostra numeros dos pontos de controle");
  printf("\n  F4: muda cor dos pontos de controle");
  printf("\n  Ctrl-Fn: desloca para baixo o n-esimo ponto de controle");
  printf("\n  Alt-Fn: desloca para cima o n-esimo ponto de controle");
  printf("\n\nPressione qualquer tecla para continuar\n\n");
  getch();

}

/* ------------------------------------------------------------------------ */

void main(void) {

  int eixos = 1, numeros = 1;
  char ch;

  nu = 35, nv = 35, inc = 0.09, TAM = 40;
  MensagemInicial();
  IniciaModoGrafico();
  DeterminaPlanoDeProjecao();
  A = getmaxx()/2; B = getmaxy()/2;

  /* ----- Definicao dos pontos de controle ----- */
  /* ----- */  m = 2; n = 2;
  /* ----- */  DEF_PONTO(p1, 0, 0, -5, -5, 1);   /* p1[0][0] = (-5, -5, 1) */
  /* ----- */  DEF_PONTO(p1, 0, 1, -5, 0, 1);
  /* ----- */  DEF_PONTO(p1, 0, 2, -5, 5, 1);
  /* ----- */  DEF_PONTO(p1, 1, 0, 0, -5, 1);
  /* ----- */  DEF_PONTO(p1, 1, 1, 0, 0, 11);
  /* ----- */  DEF_PONTO(p1, 1, 2, 0, 5, 1);
  /* ----- */  DEF_PONTO(p1, 2, 0, 5, -5, 1);
  /* ----- */  DEF_PONTO(p1, 2, 1, 5, 0, 1);
  /* ----- */  DEF_PONTO(p1, 2, 2, 5, 5, 1);
  /* -------------------------------------------- */

  do {
    setcolor(cor1);
    DesenhaSuperficie2(f1, f2, f3, 0, 1, 0, 1);
    if (eixos) {setcolor(WHITE); DesenhaEixos(3, GREEN); }
    if (numeros) {
      setcolor(YELLOW);
      numeros = MostraNumeros(p1, m, n);
    }
    else {
      outtextxy(1, 3*textheight("["),
		   "\x10 Pressione F2 para ver os numeros dos pontos");
      MarcaPontosDeControle(p1, m, n);
    }
    ch = getch();
    if (ch == 0) ch = getch();
    switch (ch) {
      case SetaPCima: if (b > 40) b -= 40; break;
      case SetaPBaixo: b += 40; break;
      case SetaPDireita:  a += 40; break;
      case SetaPEsquerda: if (a > 40) a -= 40; break;
      case PgUp: TAM += 5; break;
      case PgDn: if (TAM > 5) TAM -= 5; break;
      case TeclaF1: eixos ^= 1; break;
      case TeclaF2: numeros ^= 1; break;
      case TeclaF3: cor1 = (cor1 + 1) % 16; break;
      case TeclaF4: cor2 = (cor2 + 1) % 16; break;
      case AltF1:  (Z_ELEM(p1, 0, 0))++; break;
      case CtrlF1: (Z_ELEM(p1, 0, 0))--; break;
      case AltF2:  (Z_ELEM(p1, 0, 1))++; break;
      case CtrlF2: (Z_ELEM(p1, 0, 1))--; break;
      case AltF3:  (Z_ELEM(p1, 0, 2))++; break;
      case CtrlF3: (Z_ELEM(p1, 0, 2))--; break;
      case AltF4:  (Z_ELEM(p1, 1, 0))++; break;
      case CtrlF4: (Z_ELEM(p1, 1, 0))--; break;
      case AltF5:  (Z_ELEM(p1, 1, 1))++; break;
      case CtrlF5: (Z_ELEM(p1, 1, 1))--; break;
      case AltF6:  (Z_ELEM(p1, 1, 2))++; break;
      case CtrlF6: (Z_ELEM(p1, 1, 2))--; break;
      case AltF7:  (Z_ELEM(p1, 2, 0))++; break;
      case CtrlF7: (Z_ELEM(p1, 2, 0))--; break;
      case AltF8:  (Z_ELEM(p1, 2, 1))++; break;
      case CtrlF8: (Z_ELEM(p1, 2, 1))--; break;
      case AltF9:  (Z_ELEM(p1, 2, 2))++; break;
      case CtrlF9: (Z_ELEM(p1, 2, 2))--; break;
    }
    cleardevice();
  } while (ch != ESC);

  closegraph();
  ImprimePontosCtrl(p1, m, n);
}

/*** FIM DE BEZ_DEMO.C ***/

